/*
 * This proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 * (C) COPYRIGHT 2007 - 2009 ARM Limited
 * ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 */


#ifdef GL_ES
precision mediump float;
#endif


varying vec3 oPos;
varying vec3 tViewDir;
varying vec2 outTexcoord;

varying vec3 oLight0Dir;
varying vec3 tLight0Dir;
varying vec3 oLight1Dir;
varying vec3 tLight1Dir;

uniform sampler2D diffuseMap;
uniform sampler2D normalMap;
uniform sampler2D glossMap;

uniform vec3 cameraPosition;
uniform vec3 light0color;
uniform vec3 light1color;
uniform vec3 light2color;
uniform vec3 light0position;

const float pi = 3.1415;

#if 0
float intersectRayCylinder(vec2 ray_origin, vec2 ray_dir, vec2 cyl_pos, float cyl_radius)
{
	vec2 ray_dir2 = normalize(ray_dir);
	ray_origin -= cyl_pos;
	float a = 1.0; // dot(ray_dir2, ray_dir2);
	float b = 2.0 * dot(ray_origin, ray_dir2);
	float c = dot(ray_origin, ray_origin) - cyl_radius;
	
	float d = b - 4.0*a*c/b;
	d *= b;
	
	float t = (-b - sqrt(d));
	
	d *= 0.01;
	d = clamp(d, 0.0, 1.0);
	
	if (t < 0.0) d = 0.0;
	if (t > 2.0*length(ray_dir)) d = 0.0;
	
	return d;
}
#else

uniform vec3 shadow_info[2];
const float soft_shadow_eps = 0.015;

float intersectRayCylinder(vec2 ray_origin, vec3 shadow_info)
{
	const float scale_factor = (0.5/pi);
	
	float offset     = shadow_info.x;
	float light_dist = shadow_info.y;
	float limit = shadow_info.z;

	vec2 light_pixel = ray_origin.xy - light0position.xy;
	float angle = fract(atan(light_pixel.y, light_pixel.x)*scale_factor - offset);
	float adjusted_angle = abs(angle - 0.5);

	return smoothstep(limit, limit+soft_shadow_eps, adjusted_angle) * float(light_dist < length(light_pixel));
}
#endif

float getShadow(vec2 oPos)
{
	float shadow = 0.0;
	shadow = max(shadow, intersectRayCylinder(oPos, shadow_info[0]));
	shadow = max(shadow, intersectRayCylinder(oPos, shadow_info[1]));
	shadow = 1.0 - shadow;
	
	shadow = max(0.0, shadow);
	shadow = min(1.0, shadow);
	
	return shadow;
}

vec2 calcLight(vec3 tLightDir, vec3 oLightDir, vec3 oPos, vec3 tNormal, vec3 tRefViewDir, vec2 texcoord)
{
	float lightDist = length(tLightDir);
	tLightDir = normalize(tLightDir);
	
	float l_dot_n = max(dot(tNormal,     tLightDir), 0.0);
	float h_dot_n = max(dot(tRefViewDir, tLightDir), 0.0);
	
	float attenuation = max(0.0, 120.0 - lightDist) / 90.0;
	attenuation *= attenuation;

	float shadow = getShadow(oPos.xy) * attenuation;
	float diffuse = l_dot_n * shadow;
	float specular = pow(h_dot_n, 50.0) * shadow;
	
	return vec2(diffuse, specular);
}

void main(void)
{
#if 1
	vec2 texcoord = outTexcoord.xy;
	vec3 tNormal = normalize((2.0*texture2D(normalMap, outTexcoord.xy).xyz)-1.0);
	
	vec3 tViewDirNorm = normalize(tViewDir);
	vec3 tRefViewDir  = reflect(tViewDirNorm, tNormal);

	vec3 diffuseLight = vec3(0.0, 0,0);
	vec3 specularLight = vec3(0,0,0);

	{
		vec3 lightColor = light0color;
		vec2 diffuseSpecular = calcLight(tLight0Dir, oLight0Dir, oPos, tNormal, tRefViewDir, texcoord);
		diffuseLight.xyz += lightColor * diffuseSpecular.x;
		specularLight.xyz += lightColor * diffuseSpecular.y * texture2D(glossMap, texcoord).x;
	}
/*
	{
		vec3 lightColor = light1color;
		vec2 diffuseSpecular = calcLight(tLight1Dir, oLight1Dir, oPos, tNormal, tRefViewDir, texcoord);
		diffuseLight.xyz += lightColor * diffuseSpecular.x;
		specularLight.xyz += lightColor * diffuseSpecular.y * texture2D(glossMap, texcoord).x;
	}
*/
	
	vec3 texDiffuse = texture2D(diffuseMap, outTexcoord.xy).xyz;
	gl_FragColor = vec4(diffuseLight*texDiffuse + specularLight,1); 
#else
//	gl_FragColor = vec4(getShadow(oPos.xy, oLight0Dir.xy), 0, 0, 1);
#endif
} 
